<?php
/*
 * index.php - generate an html page of thumbnails for images and
 * files in the current directory and subdirectories.
 *
 * copy into a directory under the webroot and configure the entries
 * in the following customization section.
 *
 * part of the gallery six package:  http://muted.org/gallerysix
 *
 * Copyright (C) 2001  seth ladygo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: index.php,v 1.12 2004/12/28 01:56:17 alx Exp $
 */

// CUSTOMIZATION SECTION

// largest dimension of thumbnails, either width or height.
$max_size = 75;

// number of columns of thumbnails in the index page table.
$columns = 4;

// maximum number of entries (images and files) per page.
// any more, and we break them into smaller pages.  set to zero for an
// unlimited number of items per page.
$max_entries = 0;

// border width around thumbnails.  a border of one looks nice, but
// you may not think so.  icons are not affected and always have a
// zero width border.
$border = 1;

// do we want a formatted version of the filename shown below the
// image?
$show_filename = true;

// show formatted filenames for non-image files?
// false means show the filename as-is.
$pretty_filenames = false;

// true if the webserver allows us to refer to php files without the
// extention.  as "index" instead of "index.php", for instance.  if
// so, we can create nicer looking urls.
$drop_ext = true;

// a directory in the same directory as this script, where icons and
// files generated by this script live.  this directory must exist and
// be writable by the webserver user.
$workplace = "thumbs";

// you can put a page in the same directory as this that accepts the
// image to display as $PATH_INFO.  name that file here.  if this file
// is not found, the image is linked to directly.
$viewpage = "view.php";

// the edit page, if present in the directory, enables an edit button
// on the index page.  the edit page provides a password protected way
// to add little descriptions of the images to the index and view
// pages.
$editpage = "edit.php";

// END OF CUSTOMIZATION

// this is the class of all thumbnail entries.  most fields are
// available for use in the generated html.
class Entry {
	var $name;					// friendly filename
	var $path;					// filesystem path to the file
	var $loc;					// relative url to the file
	var $size;					// friendly file size: '1.2k'
	var $height;				// image height, in pixels
	var $width;					// image width, in pixels
	var $html_dims;				// html-friendly size: 'height=12 width=22'
	var $ctime;					// friendly date of file creation: '12 Mar 01'
	var $link;					// url link to the file
	var $border = 0;			// image link border size

	var $thumb_loc;				// relative url of the thumbnail icon
	var $thumb_html_dims;		// html-friendly thumbnail image size

	var $blurb;					// description of the file, or undefined
	var $blurb_loc;				// relative url of the blurb file
	var $blurb_var;				// name of the form variable representing
								// the blurb
	var $is_dir;				// is the entry a directory?
}


// check for GD installation
if (!function_exists('imagecreate')) {
    die('thumbdex requires PHP to be compiled with GD library support');
}

// names of the index, view, and edit pages as they appear in urls.
$indexlink = basename(__FILE__);
$viewlink = $viewpage;
$editlink = $editpage;

if ($drop_ext) {
	$indexlink = preg_replace("/^(.*)\.[a-zA-Z0-9]+/", "$1", $indexlink);
	$viewlink = preg_replace("/^(.*)\.[a-zA-Z0-9]+/", "$1", $viewlink);
	$editlink = preg_replace("/^(.*)\.[a-zA-Z0-9]+/", "$1", $editlink);
}

// in our terminology, "dir" is a filesystem path,
// "loc" is a web path.

$base_dir = dirname(__FILE__);
$base_loc = dirname(getenv('SCRIPT_NAME'));

// true if we have the ability to edit blurbs
$edit_mode = is_file("$base_dir/$editpage");

// subdir is the directory component under our root that we're looking
// at, editing, or whatever.
$subdir = preg_replace("/\/\//", "/", getenv('PATH_INFO'));
if ("/" == $subdir) $subdir = "";
// it may include a filename portion if this is called from
// view.php.  strip it if necessary.
if (!is_dir($base_dir . $subdir)) $subdir = dirname($subdir);

$work_dir = "$base_dir/$workplace";
$work_loc = "$base_loc/$workplace";

$current_loc = $base_loc . $subdir;
// friendly version of our loc
$dir_name = format_location($current_loc);

$thumb_dir = $work_dir . $current_loc;
$thumb_loc = $work_loc . $current_loc;

// return a friendlier version of this directory's name
function format_location($loc) {
	$loc = preg_replace("/^\/(.*)/", "$1", $loc);
	$loc = preg_replace("/(.*)\/$/", "$1", $loc);
	$loc = preg_replace("/\//", " : ", $loc);
	return preg_replace("/_/", " ", $loc);
}

// return a more friendly version of a filename
function format_name($filename) {
	$filename = preg_replace("/^(.*)\.[a-z]{2,4}/i", "$1", $filename);
	return preg_replace("/_/", " ", $filename);
}

// return a human-friendly version of a date
function format_date($filemod) {
	return date("d M y", $filemod);
}

// return true if we should consider this file "visible"
// in the index.  invisible files are those that start
// with a dot, are in the format of vi or emacs
// backup files (foo.txt~ or #foo.txt*), or are used
// by thumbdex (index.php, view.php).
function is_visible($dir, $file) {
	global $viewpage, $editpage, $workplace;

	return (!preg_match("/(^\.|~$|^#.*#$)/", $file) &&
		!preg_match("/(\.old$|\.bak$)/", $file) &&
		fileperms($dir ."/".$file) & 0x0004 &&
			$file != basename(__FILE__) &&
			$file != $viewpage &&
			$file != $workplace &&
			$file != $editpage);
}

// create a directory, making parent dirs if necessary
function mkdirpath($path, $mode) {
	if (strlen($path) == 0)
		return 0;

	if (strlen($path) < 3 || is_dir($path) || dirname($path) == $path)
		return 1;

	return mkdirpath(dirname($path), $mode) and mkdir($path, $mode);
}

// return a mangled name for the form variable used for this filename
function make_blurb_var($file) {
	return preg_replace("/^([^\.]+)(\.[a-z]{2,4})?$/i", "text_$1", $file);
}

// return the filename in which is stored the blurb contents
function make_blurb_filename($file) {
	return preg_replace("/^([^\.]+)(\.[a-z]{2,4})?$/i", "$1.txt", $file);
}

function escape_quotes($string) {
	return preg_replace("/\"/", "&quot;", $string);
}

// return a url for this page, paged to the given number
function get_page_link($num) {
	$url = getenv('REQUEST_URI');
	$url = preg_replace("/(\?.*)?$/", "", $url);
	return $num > 1 ? "$url?page=$num" : $url;
}

// return the current page number, or 0 if not paging
function get_current_page() {
	$page = isset($_POST['page']) ? $_POST['page'] : 0;
	$qpage = isset($_POST['qpage']) ? $_POST['qpage'] : 0;

	if ($page > 1) {
	   return $page;
	} else if ($qpage > 1) {
		return $qpage;
	} else {
		return 0;
	}
}


// for the page description blurb
$page_blurb_var = "text_page_desc";
$page_blurb_loc = "$thumb_dir/page_desc.txt";

$page_blurb = is_file($page_blurb_loc) ? join("", file($page_blurb_loc)) : "";


// icons
$absent_pic = new Entry;
$file_pic = new Entry;
$parent_pic = new Entry;
$folder_pic = new Entry;

$edit_icon = "edit.gif";
$absent_icon = "absent.gif";
$file_icon = "file.gif";
$parent_icon = "parent.gif";
$folder_icon = "folder.gif";

$edit_loc = "$work_loc/$edit_icon";

$absent_pic->thumb_loc = "$work_loc/$absent_icon";
$file_pic->thumb_loc = "$work_loc/$file_icon";
$parent_pic->thumb_loc = "$work_loc/$parent_icon";
$dir_pic->thumb_loc = "$work_loc/$folder_icon";

$absent_icon_dir = "$work_dir/$absent_icon";
$file_icon_dir = "$work_dir/$file_icon";
$parent_icon_dir = "$work_dir/$parent_icon";
$dir_icon_dir = "$work_dir/$folder_icon";

if (!is_file($absent_icon_dir)) {
	die("can't find \"absent\" icon: $absent_icon_dir");
}

if (!is_file($parent_icon_dir)) {
    die("can't find \"parent\" icon: $parent_icon_dir");
}

if (!is_file($dir_icon_dir)) {
    die("can't find \"dir\" icon: $dir_icon_dir");
}

if (!is_file($file_icon_dir)) {
    die("can't find \"file\" icon: $file_icon_dir");
}

$size = GetImageSize($absent_icon_dir);
$absent_pic->thumb_dims = $size[3];

$size = GetImageSize($parent_icon_dir);
$parent_pic->thumb_dims = $size[3];

$size = GetImageSize($dir_icon_dir);
$dir_pic->thumb_dims = $size[3];

$size = GetImageSize($file_icon_dir);
$file_pic->thumb_dims = $size[3];

// keeps track of the total number of visible files
$total_entries = 0;
$page = get_current_page();
$start_entry = ($page - 1) * $max_entries;

// scan the current directory for files, return a list.  if we're
// given an argument, return only an entry for a file by that name.
function get_entries($search_filename = "BLAHBLAHBLAH") {
	global $work_dir, $work_loc, $max_size, $columns, $border;
	global $absent_pic, $file_pic, $parent_pic, $dir_pic, $viewpage, $viewlink;
	global $current_loc, $thumb_dir, $thumb_loc, $subdir, $base_loc;
	global $indexlink, $start_entry, $max_entries;
	global $total_entries, $pretty_filenames;
	$pics = null;
	$dirs = null;
	

	$dirname = dirname(__FILE__) . $subdir;
	$dir = opendir($dirname);

	$show_parent = preg_match("/\w+/", $subdir);

	if ($show_parent && ($search_filename == "BLAHBLAHBLAH")) {
		// add a parent dir icon
		$pic = new Entry;

		$pic->name = "parent";
		$pic->link = preg_replace("/\/\//", "/", "$base_loc/$indexlink/" . dirname($subdir));
		$pic->loc = preg_replace("/\/\//", "/", "$base_loc/$subdir");
		$pic->path = "..";
		$pic->thumb_loc = $parent_pic->thumb_loc;
		$pic->thumb_html_dims = $parent_pic->thumb_dims;

		$pic->blurb_var = "text_parent_dir";
		$pic->blurb_loc = "$thumb_dir/parent_dir.txt";

		if (is_file($pic->blurb_loc))
			$pic->blurb = join("", file($pic->blurb_loc));

		$pic->is_dir = true;

		$dirs[] = $pic;
	}

	while ($file = readdir($dir))
		$files[] = $file;

	asort($files);

	$index = 0;
	foreach ($files as $file) {
		if ($search_filename != "BLAHBLAHBLAH" && $search_filename != $file)
			continue;

		if (is_visible($dirname, $file)) {
			$dirfile = "$dirname/$file";

			if ($search_filename == "BLAHBLAHBLAH" && is_file($dirfile)) {
				// spin if we've created enough entries.  we don't stop
				// because we need tp keep updating the total entries
				// counter.
				if ($max_entries > 0 && count($pics) >= $max_entries) {
					++$total_entries;
					continue;
				}

				// skip to the first entry to display
				if ($index++ < $start_entry) {
					++$total_entries;
					continue;
				}
			}

			$pic = new Entry;

			$pic->path = $dirfile;
			$pic->loc = preg_replace("/\/\//", "/", "$base_loc/$subdir/$file");
			$pic->link = preg_replace("/\/\//", "/", "$base_loc/$subdir/$file");
			$pic->name = format_name($file);
			$pic->ctime = format_date(filectime($dirfile));
			$stats = stat($dirfile);
			$pic->size = sprintf("%.1fk", $stats[7] / 1024);

			$pic->blurb_var = make_blurb_var($file);
			$pic->blurb_loc = "$thumb_dir/" . make_blurb_filename($file);

			if (is_file($pic->blurb_loc))
				$pic->blurb = join("", file($pic->blurb_loc));

			if (is_dir($dirfile)) {
				$pic->link = preg_replace("/\/\//", "/", "$base_loc/$indexlink/$subdir/$file");
				$pic->thumb_loc = $dir_pic->thumb_loc;
				$pic->thumb_html_dims = $dir_pic->thumb_dims;

				$pic->is_dir = true;

				$pic->blurb_loc = "$thumb_dir/" . make_blurb_filename($file);

				if (is_file($pic->blurb_loc))
					$pic->blurb = join("", file($pic->blurb_loc));

				// size is the number of files in the dir
				$count = 0;
				$handle = opendir($dirfile);
				while (false !== ($file = readdir($handle))) {
					if (is_visible($dirfile, $file)) {
						++$count;
					}
				}
				closedir($handle);

				$pic->size = "$count files";

				$dirs[] = $pic;
				continue;  // early out to not add to $pics
			} elseif ($size = @GetImageSize($pic->path)) {
				$pic->width = $size[0];
				$pic->height = $size[1];
				$pic->html_dims = $size[3];
				$pic->border = $border;

				// images get a viewpage link
				if (is_file("$viewpage")) {
						$pic->link = preg_replace("/\/\//", "/", "$base_loc/$viewlink/$subdir/$file");
					$pagenum = get_current_page();
					if ($pagenum > 1)
						$pic->link .= "?page=$pagenum";
				}

				// look for an existing thumbnail that's more recent than the image.
				$thumbfile = "th_$file";
				$thumbfile = preg_replace("/^(.*\.)(gif|jp(e)?g)$/i", "$1png", $thumbfile);

				$pic->thumb_loc = "$thumb_loc/$thumbfile";

				$thumbdirfile = "$thumb_dir/$thumbfile";

				// check for existing thumbnail dir
				if (!is_dir($thumb_dir)) {
					umask(002);
					if (!mkdirpath($thumb_dir, 0755)) {
						die("error creating thumbnail dir: $thumb_dir");
					}
				}

				$filemod = filemtime($dirfile);
				if (!file_exists($thumbdirfile) || filemtime($thumbdirfile) < $filemod) {
					// thumbnail needs updating
					if (preg_match("/\.jp(e)?g$/i", $file)) {
						$im = ImageCreateFromJPEG($dirfile);
					} else if (preg_match("/\.png$/i", $file)) {
						$im = ImageCreateFromPNG($dirfile);
					} else if (preg_match("/\.gif$/i", $file)) {
						if (function_exists('ImageCreateFromGIF'))
							$im = ImageCreateFromGIF($dirfile);
					}

					if ($im) {
						if ($pic->width <= $pic->height) {
							$new_h = $max_size;
							$new_w = ($pic->width / $pic->height) * $max_size;
						} else {
							$new_w = $max_size;
							$new_h = ($pic->height / $pic->width) * $max_size;
						}

						$dst_img = ImageCreate($new_w, $new_h);
						ImageCopyResized($dst_img, $im, 0, 0, 0, 0, $new_w, $new_h, $pic->width, $pic->height);

						umask(002);
						ImagePNG($dst_img, $thumbdirfile);

						$pic->thumb_html_dims = "height=\"$new_h\" width=\"$new_w\"";
					} else {
						$pic->thumb_loc = $absent_icon_loc;
						$pic->thumb_html_dims = $absent_icon_dims;
					}
				} else {
					// thumb already exists
					$thumbsize = GetImageSize($thumbdirfile);
					$pic->thumb_html_dims = $thumbsize[3];
				}
			} else {
				// some random file
				if (!$pretty_filenames)
				   $pic->name = $file;  // don't format name
				$pic->thumb_loc = $file_pic->thumb_loc;
				$pic->thumb_html_dims = $file_pic->thumb_dims;
			}

			$pics[] = $pic;
			++$total_entries;
		}
	}

	closedir($dir);

	$return[] = $pics;
	$return[] = $dirs;

	return $return;
}

// indexpage will be defined already if we are being included from
// another page.  in that case, opt out early.  we're only being used
// for our config, funcs, and data structures.  sigh.
if (isset($indexpage) and $indexpage) return;

?>

<html>
<head><title><?php print $dir_name ?></title>
<link rel="stylesheet" type="text/css" href="<?php print $work_loc ?>/style.css">
</head>
<body>

<pre>

</pre>

<table align=center cellspacing=0 cellpadding=8 width="100%">

<tr>
<td align=center valign=center><b><font class=header><?php print $dir_name ?></font></b></td>
</tr>

<tr>
<td align=center valign=center>
<?php if (strlen($page_blurb) > 0) print "$page_blurb\n"; ?>
</td>
</tr>

</table>

<pre>

</pre>

<?php
//
$return = get_entries();
$entries = $return[0];
$dirs = $return[1];

if (count($entries) > 0) {
?>

<!-- setup entry table html -->
<center>
<table cellspacing=0 cellpadding=8 width="100%">

<tr>
<td colspan="<?php print $columns ?>" align=left valign=bottom>
<font class="title">files</font>
</td>
</tr>

<?php
// build html
$count = 0;
	$cellwidth = 100 / $columns;

	foreach ($entries as $pic) {
		if ($count % $columns == 0) {
	?>
<!-- start table row html -->
<tr>

	 <?php
	}

	?>
<!-- html for each thumbnail goes here -->

<td align=center valign=top width="<?php print $cellwidth ?>%">
<a href="<?php print $pic->link ?>"><img <?php print $pic->thumb_html_dims ?> src="<? print $pic->thumb_loc ?>" border=<?php print $pic->border ?> alt="<?php print $pic->name ?>"></a><br>
<?php if ($show_filename) print "<b>$pic->name </b><br>" ?>
<font class="caption"><?php print $pic->blurb ?></font>
</td>

	<?php
	// end of row or end of last row
	if ($count % $columns == $columns - 1 || $count == count($entries) - 1) {
	?>
<!-- html for the end of a table row -->
</tr>

	<?php
		}
		++$count;
	}


	// the page navigation links
	if ($max_entries > 0 && $max_entries < $total_entries) {
		print "<tr><td align=center colspan=\"$columns\"><font class=\"pager\">page: ";

		$pagenum = 1;
		// viewed
		for (; ($pagenum - 1) * $max_entries < $start_entry; ++$pagenum) {
			print "<a class=pager href=\"". get_page_link($pagenum) . "\">$pagenum</a> ";
		}
		// current
		print  $pagenum++ . " ";

		// to view
		for (; ($pagenum - 1) * $max_entries < $total_entries; ++$pagenum) {
			print "<a class=pager href=\"". get_page_link($pagenum) . "\">$pagenum</a> ";
		}
	}

	print "</font></td></tr>\n";
?>

<!-- end entry table html -->
</table>
</center>

<pre>
</pre>

<?php 
} // end if we have entries
?>

<!-- setup directory table html -->
<center>
<table cellspacing=0 cellpadding=8 width="100%">

<tr>
<td colspan="<?php print $columns ?>" align=left valign=bottom>
<font class="title">galleries</font>
</td>
</tr>

<?php
// build html
$count = 0;

if ($dirs) {
	$cellwidth = 100 / $columns;

foreach ($dirs as $pic) {
	if ($count % $columns == 0) {
	?>
<!-- start table row html -->
<tr>

   <?php
   }

	?>
<!-- html for each directory goes here -->

<td align=center valign=bottom width="<?php print $cellwidth ?>%">
<a href="<?php print $pic->link ?>"><img <?php print $pic->thumb_html_dims ?> src="<? print $pic->thumb_loc ?>" border=<?php print $pic->border ?> alt="<?php print $pic->name ?>"></a><br>
<b><?php print $pic->name ?></b><br>
<font size="-1">
<?php if ($pic->is_dir) print $pic->size . "&nbsp;<br>" ?>
<font style="hilight"><?php print $pic->blurb ?></font>&nbsp;
</font>
</td>

	<?php
	// end of row or end of last row
	if ($count % $columns == $columns - 1 || $count == count($entries) - 1) {
	?>
<!-- html for the end of a table row -->
</tr>

	<?php
	}
	++$count;
}
}
?>

<!-- end directory table html -->
</table>
</center>


<?php
// edit icon
if ($edit_mode) {
?>

<pre>




</pre>

<!-- edit link -->
<table class="simple" border=0 align=right><tr><td class="simple" align=right>
<form method="post" action="<?php print "$base_loc/$editlink$subdir" ?>">
<?php
$pagenum = get_current_page();
if ($pagenum > 1) print "<input type=hidden name=page value=$pagenum>";
?>

<input type="image" border=0 src="<?php print $edit_loc ?>" value="edit">
</form>
</td></tr>
</table>

<?php }
?>

<!--phpinfo()-->
</body>
</html>
